import asyncio
import socket
from concurrent.futures import ThreadPoolExecutor
from unittest.mock import MagicMock, patch

import pytest

from announce_server import get_ip_address


@pytest.mark.asyncio
async def test_get_ip_address():
    with patch("socket.socket") as mock_socket:
        # Create a MagicMock object for the socket object
        mock_socket_instance = MagicMock()
        mock_socket.return_value = mock_socket_instance

        # Define the expected IP address
        expected_ip = "192.168.1.100"

        # Configure the mock socket instance to return the expected IP address
        mock_socket_instance.getsockname.return_value = (expected_ip, 0)

        # Test the get_ip_address function
        # result_ip = await asyncio.to_thread(get_ip_address)
        # loop = asyncio.get_event_loop()
        # with ThreadPoolExecutor() as pool:
        #     result_ip = await loop.run_in_executor(pool, get_ip_address)

        result_ip = get_ip_address()

        # Check if the result matches the expected IP address
        assert result_ip == expected_ip

        # Check if the socket object was created with the correct arguments
        mock_socket.assert_called_once_with(socket.AF_INET, socket.SOCK_DGRAM)

        # Check if the socket.connect method was called with the correct arguments
        mock_socket_instance.connect.assert_called_once_with(("10.255.255.255", 1))

        # Check if the socket.close method was called
        mock_socket_instance.close.assert_called_once()



from unittest.mock import MagicMock, patch

import pytest

from announce_server.decorator import (
    announce_server,
    register_block,
    register_server,
    register_service,
)


@pytest.mark.parametrize(
    "decorator_alias",
    [
        register_block,
        register_server,
    ],
)
def test_alias_calls_register_service(decorator_alias):
    test_args = (None, None)
    test_kwargs = {
        "name": "test_server",
        "ip": "127.0.0.1",
        "port": 8000,
        "host_ip": "127.0.0.1",
        "host_port": 5000,
    }

    with patch("announce_server.decorator.register_service") as mock_register_service:
        mock_register_service.return_value = MagicMock()

        decorator = decorator_alias(*test_args, **test_kwargs)
        decorator(MagicMock())

        mock_register_service.assert_called_once_with(*test_args, **test_kwargs)


@pytest.mark.parametrize(
    "decorator_alias",
    [
        announce_server,
    ],
)
def test_deprecated_alias_calls_register_service(decorator_alias):
    test_args = (None, None)
    test_kwargs = {
        "name": "test_server",
        "ip": "127.0.0.1",
        "port": 8000,
        "host_ip": "127.0.0.1",
        "host_port": 5000,
    }
    with patch("announce_server.decorator.register_service") as mock_register_service:
        mock_register_service.return_value = MagicMock()

        with pytest.warns(DeprecationWarning):
            decorator = decorator_alias(*test_args, **test_kwargs)

        decorator(MagicMock())
        mock_register_service.assert_called_once_with(*test_args, **test_kwargs)



import asyncio
import warnings
from functools import wraps

import socketio

sio = socketio.AsyncClient()


async def _announce_server(**kwargs):
    SERVER_NAME = kwargs.get("name", "server_1")
    SERVER_IP = kwargs.get("ip", "localhost")
    SERVER_PORT = kwargs.get("port", 8000)
    HOST_SERVER_IP = kwargs.get("host_ip", "0.0.0.0")
    HOST_SERVER_PORT = kwargs.get("host_port", 5000)
    RETRY_INTERVAL = kwargs.get("retry_interval", 5)

    @sio.event
    async def connect():
        await sio.emit(
            "register", {"name": SERVER_NAME, "ip": SERVER_IP, "port": SERVER_PORT}
        )
        print("Announced server to host")

    async def main():
        # retry until we connect to the host
        while True:
            try:
                await sio.connect(f"http://{HOST_SERVER_IP}:{HOST_SERVER_PORT}")
                break
            except Exception as e:
                print(e)
                print(f"Failed to connect to host, retrying in {RETRY_INTERVAL} seconds")
                await asyncio.sleep(RETRY_INTERVAL)
        # await sio.connect(f'http://{HOST_SERVER_IP}:{HOST_SERVER_PORT}')
        print("Connected to host")

    @sio.on("heartbeat")
    async def on_heartbeat():
        print("Received heartbeat from host")

    @sio.event
    async def disconnect():
        print("Disconnected from host")

    await main()


def register_service(task=None, **outer_kwargs):
    if task is None:
        return lambda f: register_service(f, **outer_kwargs)

    @wraps(task)
    def wrapper(*args, **kwargs):
        async def main(*args, **kwargs):
            loop = asyncio.get_event_loop()
            host_block_thread = loop.run_in_executor(None, task)

            # Announce the server to the host
            await _announce_server(**outer_kwargs)

            # Wait for host_block to finish
            await host_block_thread

        return asyncio.run(main())

    return wrapper


def announce_server(*args, **kwargs):
    """Wrapper for register_service"""
    warnings.warn(
        "announce_server is deprecated, use register_service instead",
        DeprecationWarning,
        stacklevel=2,
    )
    return register_service(*args, **kwargs)


def register_block(*args, **kwargs):
    """Wrapper for register_service"""
    return register_service(*args, **kwargs)


def register_server(*args, **kwargs):
    """Wrapper for register_service"""
    return register_service(*args, **kwargs)
